<?php
namespace tmc\revisionmanager\src\AdminPages\PageSettings;

/**
 * @author jakubkuranda@gmail.com
 * Date: 15.03.2018
 * Time: 14:14
 */

use shellpress\v1_3_82\src\Shared\AdminPageFramework\AdminPageTab;
use tmc\revisionmanager\src\App;
use TMC_v1_0_6_AdminPageFramework;

class TabTools extends AdminPageTab {

    /**
     * Declaration of current element.
     */
    public function setUp() {

        //  ----------------------------------------
        //  Definition
        //  ----------------------------------------

        $this->pageFactory->addInPageTab(
            array(
                'page_slug'     =>  $this->pageSlug,
                'tab_slug'      =>  $this->tabSlug,
                'title'         =>  __( 'Tools', 'rm_tmc' ),
            )
        );

        //  ----------------------------------------
        //  Filters
        //  ----------------------------------------

        add_filter( 'fields_' . $this->pageFactoryClassName,        array( $this, '_f_redefineFields_importexport' ) );
        add_filter( 'validation_' . $this->pageFactoryClassName,    array( $this, '_f_rollbackFactorySettings' ), 10, 4 );

    }

    /**
     * Called while current component is loaded.
     */
    public function load() {

        //  ----------------------------------------
        //  Sections
        //  ----------------------------------------

        $this->pageFactory->addSettingSections(
            array(
                'section_id'            =>  'importexport',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'order'                 =>  5,
                'title'                 =>  __( 'Import / Export', 'rm_tmc' ),
                'description'           =>  array(
                    __( 'Share your settings between many Wordpress installations ( multisite ).', 'rm_tmc' )
                )
            ),
	        array(
		        'section_id'            =>  'defaultSettings',
		        'page_slug'             =>  $this->pageSlug,
		        'tab_slug'              =>  $this->tabSlug,
		        'order'                 =>  10,
		        'title'                 =>  __( 'Default settings', 'rm_tmc' ),
		        'description'           =>  __( 'These actions can not be undone. Be careful.', 'rm_tmc' ),
		        'save'                  =>  false
	        ),
	        array(
		        'section_id'            =>  'system',
		        'page_slug'             =>  $this->pageSlug,
		        'tab_slug'              =>  $this->tabSlug,
		        'order'                 =>  15,
		        'title'                 =>  __( 'System info', 'rm_tmc' ),
		        'description'           =>  array(
		        	__( 'This data will help us debug your environment. Keep in mind, that some data may be sensitive.', 'rm_tmc' ),
		        	sprintf( '<strong>%1$s</strong>', __( 'Do not share it in public places such as forums etc.', 'rm_tmc' ) ),
		        ),
		        'save'                  =>  false
	        )
        );

        //  ----------------------------------------
        //  Fields
        //  ----------------------------------------

        $this->pageFactory->addSettingFields(
            'importexport',   //  section_id
            array(
                'field_id'              =>  'exportSettings',
                'title'                 =>  __( 'Export settings', 'rm_tmc' ),
                'type'                  =>  'export',
                'file_name'             =>  sprintf( 'Revision Manager TMC options - %1$s.txt', current_time( 'mysql' ) ),
                'data'                  =>  App::s()->options->get(),
                'label'                 =>  __( 'Download all settings', 'rm_tmc' )
            ),
            array(
                'field_id'              =>  'importSettings',
                'title'                 =>  __( 'Import settings', 'rm_tmc' ),
                'type'                  =>  'import',
                'option_key'            =>  App::s()->options->getOptionsKey(),
                'label'                 =>  __( 'Load settings from file', 'rm_tmc' ),
                'is_merge'              =>  true,
                'description'           =>  __( 'Pick file exported from other WordPress installation.', 'rm_tmc' )
            )
        );

        $this->pageFactory->addSettingFields(
        	'defaultSettings',  //  section_id
	        array(
	        	'field_id'              =>  'rollbackSubmit',
		        'type'                  =>  'submit',
		        'value'                 =>  __( 'Rollback factory settings', 'rm_tmc' ),
		        'attributes'            =>  array(
		        	'class'                 =>  'button button-primary'
		        )
	        )
        );

	    $this->pageFactory->addSettingFields(
		    'system',   //  section_id
		    array(
			    'field_id'              =>  'downloadInfo',
			    'type'                  =>  'export',
			    'data'                  =>  App::i()->system->getAllSystemInfoDump(),
			    'file_name'             =>  'RevisionManagerTMC dump - ' . current_time( 'mysql' ) . '.txt',
			    'label'                 =>  __( 'Save system info as file', 'rm_tmc' ),
			    'format'                =>  'text',
			    'before_field'          =>  sprintf( '<div><textarea readonly rows="10" class="system large-text" style="font-family: monospace;">%1$s</textarea></div>', App::i()->system->getAllSystemInfoDump() )
		    )
	    );

    }

    //  ================================================================================
    //  FILTERS
    //  ================================================================================


    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineFields_importexport( $fields ) {

        if( ! App::i()->license->isLicenseActive() ){

        	//  ----------------------------------------
        	//  Modify export field
        	//  ----------------------------------------

        	if( isset( $fields['importexport']['exportSettings'] ) ){

		        $fields['importexport']['exportSettings']['attributes']['disabled'] = 'disabled';
		        $fields['importexport']['exportSettings']['after_field'] = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
			        App::TMC_SHOP_URL,
			        __( 'You can export current settings to file in PRO version.', 'rm_tmc' )
		        );

	        }

	        //  ----------------------------------------
	        //  Modify import field
	        //  ----------------------------------------

	        if( isset( $fields['importexport']['importSettings'] ) ){

		        $fields['importexport']['importSettings']['attributes']['disabled'] = 'disabled';
		        $fields['importexport']['importSettings']['description'] = null;
		        $fields['importexport']['importSettings']['after_field'] = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
			        App::TMC_SHOP_URL,
			        __( 'You can import settings from other WP in PRO version.', 'rm_tmc' )
		        );

	        }

        }

        return $fields;

    }

	/**
	 * Rollbacks factory settings if button has been pressed.
	 *
	 * @param array $newOptions
	 * @param array $oldOptions
	 * @param TMC_v1_0_6_AdminPageFramework $apf
	 * @param array $submitInfo
	 *
	 * @return array
	 */
    public function _f_rollbackFactorySettings( $newOptions, $oldOptions, $apf, $submitInfo ) {

    	if( $submitInfo['section_id'] === 'defaultSettings' && $submitInfo['field_id'] === 'rollbackSubmit' ){
    		return App::s()->options->getDefaultOptions();
	    }

    	return $newOptions;

    }

}